// KbInput.cpp  Implements basic keyboard functions
// version 1.0
// Copyright 2005 Chad Berchek

#include <cstdio>
#include <algorithm>
#include <cstring>
#include <cctype>
#include "kbinput.h"
using namespace std;

// Returns true if string contains any characters other than space, tab, CR, or LF
bool isEmptyString(const char *str) {
	size_t len = strlen(str);
	for (size_t i = 0; i < len; i++) {
		if (str[i] != ' ' && str[i] != '\t' && str[i] != '\n' && str[i] != '\r') return false;
	}
	return true;
}

// Reads string from stdin into buffer. '\n' will not appear at the end of the string returned.
// If the input exceeds the buffer length, the remaining characters in stdin will be flushed.
// Returns the same buffer pointer that was passed in. Returns immediately if bufLen = 0.
char *kbReadString(char *buf, const unsigned long bufLen) {
	if (bufLen == 0) return buf;
	fgets(buf, bufLen, stdin);
	size_t entryLen = strlen(buf);
	if (entryLen > 0) {
		// If last char in buf is '\n', remove it.
		if (buf[entryLen - 1] == '\n') buf[entryLen - 1] = '\0';
		else kbReadEnter(); // If '\n' not at end of buf, '\n' is still in stdin so call kbReadEnter
	}
	else kbReadEnter(); // If entryLen == 0, '\n' is still in stdin
	return buf;
}

// Read characters from stdin until '\n' found.
void kbReadEnter() {
	while(getchar() != '\n');
}

// Reads a double type number from stdin. It must be in the range [min, max]. If nothing is
// entered, defaultValue is returned, unless the default is disabled. To disable the default,
// send a default that is outside of the range [min, max]. With no default, the user will be
// forced to enter some number.
double kbReadDouble(double min, double max, double defaultValue) {
	const size_t BUFLEN = 30;
	char buf[BUFLEN];

	double output;
	
	if (min > max) swap(min, max);
	
	bool defaultExists = true;
	if (defaultValue < min || defaultValue > max) defaultExists = false;
	
	do {
		kbReadString(buf, BUFLEN);
		if (defaultExists && isEmptyString(buf)) {
			output = defaultValue;
			break;
		}
		output = atof(buf);
	} while (output < min || output > max);
	
	return output;
}

// Return true if 'y' is read from stdin or false if 'n' is read (case insensitive).
// Other characters are ignored. It only returns once 'y' or 'n' is read. Any extra
// characters in stdin are removed, up through the first '\n' found.
bool kbReadYN() {
	int c;
	do {
		c = tolower(getchar());
	} while (!(c == 'y' || c == 'n'));
	kbReadEnter(); // clean up '\n' which is probably still in stdin
	return c == 'y';
}
